﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;

namespace PerlinNoiseShader.ImprovedNoise
{
    public class ImprovedNoiseClass:NoiseMask
    {
        
        private ImageBrush oG1ImageBrush;
        private ImageBrush oG2ImageBrush;
        
        private ImageBrush oPrepBrush;
         // private ImageBrush oDrawImageBrush;
        public WriteableBitmap oPrepBitmap;
        private ImprovedNoiseShaderEffect[] aImprovedNoiseShaderEffect;
        private ImprovedNoiseShaderEffect3D[] aImprovedNoiseShaderEffect3D; 

        public ImprovedNoiseClass():this((new Random()).Next())  {}
        public ImprovedNoiseClass(int random)
        {
            oDrawImage = new Image();
            iUIElement = oDrawImage;
            aImprovedNoiseShaderEffect = new ImprovedNoiseShaderEffect[4];
            aImprovedNoiseShaderEffect3D = new ImprovedNoiseShaderEffect3D[5];
            for (int i = 0; i < 4; i++)
            {
                aImprovedNoiseShaderEffect[i] = new ImprovedNoiseShaderEffect((ImprovedNoiseShaderEffect.state)i);
                aImprovedNoiseShaderEffect3D[i]=new ImprovedNoiseShaderEffect3D((ImprovedNoiseShaderEffect3D.state)i);
            }
            reInitialize(random);
            aImprovedNoiseShaderEffect3D[4] = new ImprovedNoiseShaderEffect3D((ImprovedNoiseShaderEffect3D.state)4);
        }

        

        

        private void getG()
        {
            int[][] grad3 = {new int[]{1,1,0},new int[]{-1,1,0},new int[]{1,-1,0},new int[]{-1,-1,0},
                    new int[]{1,0,1},new int[]{-1,0,1},new int[]{1,0,-1},new int[]{-1,0,-1},
                    new int[]{0,1,1},new int[]{0,-1,1},new int[]{0,1,-1},new int[]{0,-1,-1}};
            oG1ImageBrush = new ImageBrush();
            oG2ImageBrush = new ImageBrush();
            WriteableBitmap g1Bitmap = new WriteableBitmap(256, 1);
            WriteableBitmap g2Bitmap = new WriteableBitmap(256, 1);
            for (int i = 0; i < 256; i++)
            {
                int perm1 = p[i] % 12;
                int perm2 = p[(i + 1) & 255] % 12;
                int[] vector = grad3[perm1];
                int value1 = (255 << 24) | ((vector[0] + 1) << 16) | ((vector[1] + 1) << 8) | (vector[2] + 1);
                vector = grad3[perm2];
                int value2 = (255 << 24) | ((vector[0] + 1) << 16) | ((vector[1] + 1) << 8) | (vector[2] + 1);
                g1Bitmap.Pixels[i] = value1;
                g2Bitmap.Pixels[i] = value2;
            }
            g1Bitmap.Invalidate();
            g2Bitmap.Invalidate();
            oG1ImageBrush.ImageSource = g1Bitmap;
            oG2ImageBrush.ImageSource = g2Bitmap;
        }

        public void reInitialize()
        {
            reInitialize((new Random()).Next());
        }
        
        public void reInitialize(int random)
        {
            oRandom = new Random(random);
            thisSeed = random;
            oPBrush = getP();
            getG();
            Image oPrepImage = new Image();
            oPrepImage.Measure(new Size(256, 256));
            oPrepImage.Arrange(new Rect(0, 0, 256, 256));
            oPrepImage.Source = new WriteableBitmap(256, 256);
            PrepImprovedNoiseEffect oPrepImprovedNoiseEffect = new PrepImprovedNoiseEffect();
            oPrepImage.Effect = oPrepImprovedNoiseEffect;
            oPrepImprovedNoiseEffect.pArray = oPBrush;
            oPrepBitmap = new WriteableBitmap(256, 256);
            oPrepBrush = new ImageBrush();
            oPrepBitmap.Render(oPrepImage, null);
            oPrepBitmap.Invalidate();
            oPrepBrush.ImageSource = oPrepBitmap;

        }

        public void singleNoise(float z, Size freqXY, Point offset, float amplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawImage != null) createImage();
            }

            ImprovedNoiseShaderEffect oImprovedNoiseShaderEffect=absolute?aImprovedNoiseShaderEffect[1]:aImprovedNoiseShaderEffect[0];
            iUIElement.Effect = oImprovedNoiseShaderEffect;
            oImprovedNoiseShaderEffect.amplitude = absolute ? amplitude : 0.5f * amplitude;
            oImprovedNoiseShaderEffect.c1 = c1;
            oImprovedNoiseShaderEffect.c2 = c2;
            oImprovedNoiseShaderEffect.z = z;
            oImprovedNoiseShaderEffect.size = new Size(freqXY.Width * thisSize.Width, freqXY.Height * thisSize.Height);
            oImprovedNoiseShaderEffect.offsetXY = new Point(offset.X * freqXY.Width, offset.Y * freqXY.Height);
            oImprovedNoiseShaderEffect.pArray = oPrepBrush;
            oImprovedNoiseShaderEffect.g1Array = oG1ImageBrush;
            oImprovedNoiseShaderEffect.g2Array = oG2ImageBrush;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
            
        }

        public void noise(float z, Size freqXY, Point offset, bool absolute, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawImage != null) createImage();
            }

            ImprovedNoiseShaderEffect oImprovedNoiseShaderEffect = absolute ? aImprovedNoiseShaderEffect[3] : aImprovedNoiseShaderEffect[2];
            iUIElement.Effect = oImprovedNoiseShaderEffect;
            oImprovedNoiseShaderEffect.z = z;
            oImprovedNoiseShaderEffect.size = new Size(freqXY.Width * thisSize.Width, freqXY.Height * thisSize.Height);
            oImprovedNoiseShaderEffect.offsetXY = new Point(offset.X * freqXY.Width, offset.Y * freqXY.Height);
            oImprovedNoiseShaderEffect.pArray = oPrepBrush;
            oImprovedNoiseShaderEffect.g1Array = oG1ImageBrush;
            oImprovedNoiseShaderEffect.g2Array = oG2ImageBrush;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }

        public void singleNoise(ImageBrush xPlane, ImageBrush yPlane, ImageBrush zPlane, Vector3d offset, float amplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawImage != null) createImage();
            }

            ImprovedNoiseShaderEffect3D oImprovedNoiseShaderEffect3D = absolute ? aImprovedNoiseShaderEffect3D[1] : aImprovedNoiseShaderEffect3D[0];
            iUIElement.Effect = oImprovedNoiseShaderEffect3D;
            oImprovedNoiseShaderEffect3D.amplitude = absolute ? amplitude : 0.5f * amplitude;
            oImprovedNoiseShaderEffect3D.c1 = c1;
            oImprovedNoiseShaderEffect3D.c2 = c2;
            oImprovedNoiseShaderEffect3D.pArray = oPrepBrush;
            oImprovedNoiseShaderEffect3D.g1Array = oG1ImageBrush;
            oImprovedNoiseShaderEffect3D.g2Array = oG2ImageBrush;
            oImprovedNoiseShaderEffect3D.offsetXY = new Point(offset.x,offset.y);
            oImprovedNoiseShaderEffect3D.offsetZ = (float)offset.z;
            oImprovedNoiseShaderEffect3D.xArray = xPlane;
            oImprovedNoiseShaderEffect3D.yArray = yPlane;
            oImprovedNoiseShaderEffect3D.zArray = zPlane;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }

        public void noise(ImageBrush xPlane, ImageBrush yPlane, ImageBrush zPlane, Vector3d mult, Vector3d offset, bool absolute, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawImage != null) createImage();
            }

            ImprovedNoiseShaderEffect3D oImprovedNoiseShaderEffect3D = absolute ? aImprovedNoiseShaderEffect3D[3] : aImprovedNoiseShaderEffect3D[2];
            iUIElement.Effect = oImprovedNoiseShaderEffect3D;
            oImprovedNoiseShaderEffect3D.pArray = oPrepBrush;
            oImprovedNoiseShaderEffect3D.g1Array = oG1ImageBrush;
            oImprovedNoiseShaderEffect3D.g2Array = oG2ImageBrush;
            oImprovedNoiseShaderEffect3D.offsetXY = new Point(offset.x,offset.y);
            oImprovedNoiseShaderEffect3D.offsetZ = (float)offset.z;
            oImprovedNoiseShaderEffect3D.multXY = new Point(mult.x,mult.y);
            oImprovedNoiseShaderEffect3D.multZ = (float)mult.z;
            oImprovedNoiseShaderEffect3D.xArray = xPlane;
            oImprovedNoiseShaderEffect3D.yArray = yPlane;
            oImprovedNoiseShaderEffect3D.zArray = zPlane;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }
        
        public void noiseAdd(ImageBrush xPlane, ImageBrush yPlane, ImageBrush zPlane, ImageBrush input, Vector3d mult,Vector3d offset, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawImage != null) createImage();
            }

            ImprovedNoiseShaderEffect3D oImprovedNoiseShaderEffect3D = aImprovedNoiseShaderEffect3D[4];
            iUIElement.Effect = oImprovedNoiseShaderEffect3D;
            oImprovedNoiseShaderEffect3D.pArray = oPrepBrush;
            oImprovedNoiseShaderEffect3D.g1Array = oG1ImageBrush;
            oImprovedNoiseShaderEffect3D.g2Array = oG2ImageBrush;
            oImprovedNoiseShaderEffect3D.multXY = new Point(mult.x,mult.y);
            oImprovedNoiseShaderEffect3D.multZ = (float)mult.z;
            oImprovedNoiseShaderEffect3D.offsetXY = new Point(offset.x,offset.y);
            oImprovedNoiseShaderEffect3D.offsetZ = (float)offset.z;
            oImprovedNoiseShaderEffect3D.xArray = xPlane;
            oImprovedNoiseShaderEffect3D.yArray = yPlane;
            oImprovedNoiseShaderEffect3D.zArray = zPlane;
            oImprovedNoiseShaderEffect3D.input = input;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }

    }
}
