﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;

namespace PerlinNoiseShader.ImprovedNoise
{
    public class ImprovedNoiseShaderEffect3D : ShaderEffect
    {
        public enum state { single, singleAbs, standard, standardAbs, addStandardAbs };
        private DependencyProperty offsetXYProperty;
        private DependencyProperty offsetZProperty;
        private DependencyProperty pArrayProperty;
        private DependencyProperty g1ArrayProperty;
        private DependencyProperty g2ArrayProperty;
        private DependencyProperty xArrayProperty;
        private DependencyProperty yArrayProperty;
        private DependencyProperty zArrayProperty;
        private DependencyProperty inputProperty;
        private DependencyProperty multXYProperty;
        private DependencyProperty multZProperty;

        private DependencyProperty amplitudeProperty;
        private DependencyProperty c1Property;
        private DependencyProperty c2Property;
        public ImprovedNoiseShaderEffect3D() : this(state.single) { }

        public ImprovedNoiseShaderEffect3D(state eState)
        {

            pArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("pArray", typeof(ImprovedNoiseShaderEffect3D), 0);
            g1ArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("g1Array", typeof(ImprovedNoiseShaderEffect3D), 1);
            g2ArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("g2Array", typeof(ImprovedNoiseShaderEffect3D), 2);
            xArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("xArray", typeof(ImprovedNoiseShaderEffect3D), 3);
            yArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("yArray", typeof(ImprovedNoiseShaderEffect3D), 4);
            zArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("zArray", typeof(ImprovedNoiseShaderEffect3D), 5);
            inputProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("inputArray", typeof(ImprovedNoiseShaderEffect3D), 6);
            offsetXYProperty = DependencyProperty.Register("offsetXY", typeof(Point), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(new Point(0, 0), PixelShaderConstantCallback(0)));
            offsetZProperty = DependencyProperty.Register("offsetZ", typeof(float), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(0f, PixelShaderConstantCallback(1)));
            
            switch (eState)
            {
                case state.single:
                    {
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(3)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(4)));
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(1f, PixelShaderConstantCallback(2)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise/ImprovedPerlinNoise3D.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(amplitudeProperty);
                        break;
                    }
                case state.singleAbs:
                    {
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(3)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(4)));
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(1f, PixelShaderConstantCallback(2)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise/ImprovedPerlinNoise3DAbs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(amplitudeProperty);
                        break;
                    }
                case state.standard:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise/ImprovedPerlinNoise3D2.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        multXYProperty = DependencyProperty.Register("multXY", typeof(Point), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(new Point(1, 1), PixelShaderConstantCallback(2)));
                        multZProperty = DependencyProperty.Register("multZ", typeof(float), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(1f, PixelShaderConstantCallback(3)));
                        UpdateShaderValue(multXYProperty);
                        UpdateShaderValue(multZProperty);
                        break;
                    }
                case state.standardAbs:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise/ImprovedPerlinNoise3D2Abs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        multXYProperty = DependencyProperty.Register("multXY", typeof(Point), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(new Point(1, 1), PixelShaderConstantCallback(2)));
                        multZProperty = DependencyProperty.Register("multZ", typeof(float), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(1f, PixelShaderConstantCallback(3)));
                        UpdateShaderValue(multXYProperty);
                        UpdateShaderValue(multZProperty);
                        break;
                    }
                case state.addStandardAbs:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise/ImprovedPerlinNoise3DAdd.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        multXYProperty = DependencyProperty.Register("multXY", typeof(Point), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(new Point(1, 1), PixelShaderConstantCallback(2)));
                        multZProperty = DependencyProperty.Register("multZ", typeof(float), typeof(ImprovedNoiseShaderEffect3D), new PropertyMetadata(1f, PixelShaderConstantCallback(3)));
                        UpdateShaderValue(multXYProperty);
                        UpdateShaderValue(multZProperty);
                        UpdateShaderValue(inputProperty);
                        break;
                    }

            }
            UpdateShaderValue(pArrayProperty);
            UpdateShaderValue(g1ArrayProperty);
            UpdateShaderValue(g2ArrayProperty);
            UpdateShaderValue(xArrayProperty);
            UpdateShaderValue(yArrayProperty);
            UpdateShaderValue(zArrayProperty);
            UpdateShaderValue(offsetXYProperty);
            UpdateShaderValue(offsetZProperty);
        }

        public Point offsetXY
        {
            get { return (Point)GetValue(offsetXYProperty); }
            set { SetValue(offsetXYProperty, value); }
        }

        public float offsetZ
        {
            get { return (float)GetValue(offsetZProperty); }
            set { SetValue(offsetZProperty, value); }
        }

        public Brush pArray
        {
            get { return (Brush)GetValue(pArrayProperty); }
            set { SetValue(pArrayProperty, value); }
        }

        public Brush g1Array
        {
            get { return (Brush)GetValue(g1ArrayProperty); }
            set { SetValue(g1ArrayProperty, value); }
        }

        public Brush g2Array
        {
            get { return (Brush)GetValue(g2ArrayProperty); }
            set { SetValue(g2ArrayProperty, value); }
        }

        public Color c1
        {
            get { return (Color)GetValue(c1Property); }
            set { SetValue(c1Property, value); }
        }

        public Color c2
        {
            get { return (Color)GetValue(c2Property); }
            set { SetValue(c2Property, value); }
        }

        public float amplitude
        {
            get { return (float)GetValue(amplitudeProperty); }
            set { SetValue(amplitudeProperty, value); }
        }

        public Brush xArray
        {
            get { return (Brush)GetValue(xArrayProperty); }
            set { SetValue(xArrayProperty, value); }
        }

        public Brush yArray
        {
            get { return (Brush)GetValue(yArrayProperty); }
            set { SetValue(yArrayProperty, value); }
        }

        public Brush zArray
        {
            get { return (Brush)GetValue(zArrayProperty); }
            set { SetValue(zArrayProperty, value); }
        }

        public Brush input
        {
            get { return (Brush)GetValue(inputProperty); }
            set { SetValue(inputProperty, value); }
        }
        
        public Point multXY
        {
            get { return (Point)GetValue(multXYProperty); }
            set { SetValue(multXYProperty, value); }
        }

        public float multZ
        {
            get { return (float)GetValue(multZProperty); }
            set { SetValue(multZProperty, value); }
        }
    }
}
