﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;

namespace PerlinNoiseShader.PerlinNoise2D
{
    public class TileableEffect:ShaderEffect
    {
        public enum state { single, singleAbs, singleX, singleAbsX, singleY, singleAbsY, standard, standardAbs, standardX, standardAbsX, standardY, standardAbsY };

        private DependencyProperty input1Property;
        private DependencyProperty input2Property;
        private DependencyProperty input3Property;
        private DependencyProperty input4Property;
        private DependencyProperty sizeProperty;
        private DependencyProperty offsetProperty;
        private DependencyProperty amplitudeProperty;
        private DependencyProperty c1Property;
        private DependencyProperty c2Property;
        
        public TileableEffect(state eState)
        {
            input1Property = ShaderEffect.RegisterPixelShaderSamplerProperty("input1", typeof(TileableEffect), 0);
            input2Property = ShaderEffect.RegisterPixelShaderSamplerProperty("input2", typeof(TileableEffect), 1);
            input3Property = ShaderEffect.RegisterPixelShaderSamplerProperty("input3", typeof(TileableEffect), 2);
            input4Property = ShaderEffect.RegisterPixelShaderSamplerProperty("input4", typeof(TileableEffect), 3);
            sizeProperty = DependencyProperty.Register("size", typeof(Size), typeof(TileableEffect), new PropertyMetadata(new Size(640, 480), PixelShaderConstantCallback(0)));
            offsetProperty = DependencyProperty.Register("offset", typeof(Point), typeof(TileableEffect), new PropertyMetadata(new Point(0, 0), PixelShaderConstantCallback(1)));
            switch (eState)
            {
                case state.single:
                    {
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(TileableEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(2)));
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(3)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(4)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileable.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(input3Property);
                        UpdateShaderValue(input4Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        UpdateShaderValue(amplitudeProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        break;
                    }
                case state.singleX:
                    {
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(TileableEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(2)));
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(3)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(4)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileableX.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        UpdateShaderValue(amplitudeProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        break;
                    }
                case state.singleY:
                    {
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(TileableEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(2)));
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(3)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(4)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileableY.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        UpdateShaderValue(amplitudeProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        break;
                    }
                case state.singleAbs:
                    {
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(TileableEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(2)));
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(3)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(4)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileableAbs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(input3Property);
                        UpdateShaderValue(input4Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        UpdateShaderValue(amplitudeProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        break;
                    }
                case state.singleAbsX:
                    {
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(TileableEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(2)));
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(3)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(4)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileableXAbs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        UpdateShaderValue(amplitudeProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        break;
                    }
                case state.singleAbsY:
                    {
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(TileableEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(2)));
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(3)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(TileableEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(4)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileableYAbs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        UpdateShaderValue(amplitudeProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        break;
                    }
                case state.standard:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileable2.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(input3Property);
                        UpdateShaderValue(input4Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        break;
                    }
                case state.standardX:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileable2X.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        break;
                    }
                case state.standardY:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileable2Y.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        break;
                    }
                case state.standardAbs:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileable2Abs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(input3Property);
                        UpdateShaderValue(input4Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        break;
                    }
                case state.standardAbsX:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileable2XAbs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        break;
                    }
                case state.standardAbsY:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/makeTileable2YAbs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(input1Property);
                        UpdateShaderValue(input2Property);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        break;
                    }
               
            }
           
        }

        public Brush input1
        {
            get { return (Brush)GetValue(input1Property); }
            set { SetValue(input1Property, value); }
        }

        public Brush input2
        {
            get { return (Brush)GetValue(input2Property); }
            set { SetValue(input2Property, value); }
        }

        public Brush input3
        {
            get { return (Brush)GetValue(input3Property); }
            set { SetValue(input3Property, value); }
        }

        public Brush input4
        {
            get { return (Brush)GetValue(input4Property); }
            set { SetValue(input4Property, value); }
        }

        public Size size
        {
            get { return (Size)GetValue(sizeProperty); }
            set { SetValue(sizeProperty, value); }
        }

        public Point offset
        {
            get { return (Point)GetValue(offsetProperty); }
            set { SetValue(offsetProperty, value); }
        }

        public float amplitude
        {
            get { return (float)GetValue(amplitudeProperty); }
            set { SetValue(amplitudeProperty, value); }
        }

        public Color c1
        {
            get { return (Color)GetValue(c1Property); }
            set { SetValue(c1Property, value); }
        }

        public Color c2
        {
            get { return (Color)GetValue(c2Property); }
            set { SetValue(c2Property, value); }
        }
        
    }
}
