﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using PerlinNoiseShader;

namespace PerlinProject.SphereMapping
{
    public class SphereMappingLightEffect:ShaderEffect
    {

        public readonly DependencyProperty m00_m01Property = DependencyProperty.Register("m00_m01", typeof(Point), typeof(SphereMappingLightEffect), new PropertyMetadata(new Point(), PixelShaderConstantCallback(0)));
        public readonly DependencyProperty m02_m10Property = DependencyProperty.Register("m02_m10", typeof(Point), typeof(SphereMappingLightEffect), new PropertyMetadata(new Point(), PixelShaderConstantCallback(1)));
        public readonly DependencyProperty m11_m12Property = DependencyProperty.Register("m11_m12", typeof(Point), typeof(SphereMappingLightEffect), new PropertyMetadata(new Point(), PixelShaderConstantCallback(2)));
        public readonly DependencyProperty m20_m21Property = DependencyProperty.Register("m20_m21", typeof(Point), typeof(SphereMappingLightEffect), new PropertyMetadata(new Point(), PixelShaderConstantCallback(3)));
        public readonly DependencyProperty m22Property = DependencyProperty.Register("m22", typeof(float), typeof(SphereMappingLightEffect), new PropertyMetadata(0f, PixelShaderConstantCallback(4)));
        public readonly DependencyProperty sourceBitmapProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("sourceBitmap", typeof(SphereMappingLightEffect), 0);
        public readonly DependencyProperty lightBitmapProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("lightBitmap", typeof(SphereMappingLightEffect), 1);

        public SphereMappingLightEffect()
        {
            Uri u = new Uri(@"/PerlinProject;component/SphereMapping/SphereMapLight.ps", UriKind.Relative);
            PixelShader = new PixelShader() { UriSource = u };
            UpdateShaderValue(m00_m01Property);
            UpdateShaderValue(m02_m10Property);
            UpdateShaderValue(m11_m12Property);
            UpdateShaderValue(m20_m21Property);
            UpdateShaderValue(m22Property);
            UpdateShaderValue(sourceBitmapProperty);
            UpdateShaderValue(lightBitmapProperty);
        }

        public Matrix3d rotMatrix
        {
            get
            {
                Matrix3d oMatrix = new Matrix3d();
                Point tmp = (Point)GetValue(m00_m01Property);
                oMatrix.m00 = tmp.X;
                oMatrix.m01 = tmp.Y;
                tmp = (Point)GetValue(m02_m10Property);
                oMatrix.m02 = tmp.X;
                oMatrix.m10 = tmp.Y;
                tmp = (Point)GetValue(m11_m12Property);
                oMatrix.m11 = tmp.X;
                oMatrix.m12 = tmp.Y;
                tmp = (Point)GetValue(m20_m21Property);
                oMatrix.m20 = tmp.X;
                oMatrix.m21 = tmp.Y;
                oMatrix.m22 = (float)GetValue(m22Property);
                return oMatrix;
            }
            set
            {
                Matrix3d oMatrix = (Matrix3d)value;
                Point tmp = new Point(oMatrix.m00, oMatrix.m01);
                SetValue(m00_m01Property, tmp);
                tmp = new Point(oMatrix.m02, oMatrix.m10);
                SetValue(m02_m10Property, tmp);
                tmp = new Point(oMatrix.m11, oMatrix.m12);
                SetValue(m11_m12Property, tmp);
                tmp = new Point(oMatrix.m20, oMatrix.m21);
                SetValue(m20_m21Property, tmp);
                SetValue(m22Property, (float)oMatrix.m22);
            }
        }
        
       public Brush sourceBrush
        {
            get { return (Brush)GetValue(sourceBitmapProperty); }
            set { SetValue(sourceBitmapProperty, value); }
        }
        public Brush lightBrush
        {
            get { return (Brush)GetValue(lightBitmapProperty); }
            set { SetValue(lightBitmapProperty, value); }
        }
    
    }
}
