﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;

namespace PerlinNoiseShader
{
    //written by Stefan Gustavson, ported to C# by Andreas Meisel, only for tests
    
    public class ClassicNoise
    { // Classic Perlin noise in 3D, for comparison
        int[][] grad3 = {new int[]{1,1,0},new int[]{-1,1,0},new int[]{1,-1,0},new int[]{-1,-1,0},
                    new int[]{1,0,1},new int[]{-1,0,1},new int[]{1,0,-1},new int[]{-1,0,-1},
                    new int[]{0,1,1},new int[]{0,-1,1},new int[]{0,1,-1},new int[]{0,-1,-1}};
        private int[] p=new int[256];
        // To remove the need for index wrapping, double the permutation table length
        int[] perm = new int[512];
        public WriteableBitmap permBitmap;
        public ClassicNoise(int random)
        {
            Random oRandom = new Random(random);
            int i, j, k;
            for (i = 0; i < 256; i++) p[i] = i;
            while (--i > 0)
            {
                k = p[i];
                j = (int)(oRandom.Next() % 256);
                p[i] = p[j];
                p[j] = k;
            }
            for (i = 0; i < 512; i++) perm[i] = p[i & 255];
            //permBitmap = this.getPerm();
        }

        // This method is a *lot* faster than using (int)Math.floor(x)
        private int fastfloor(double x)
        {
            return x > 0 ? (int)x : (int)x - 1;
        }
        private double dot(int[] g, double x, double y, double z)
        {
            return g[0] * x + g[1] * y + g[2] * z;
        }
        private static double mix(double a, double b, double t)
        {
            return (1 - t) * a + t * b;
        }
        private static double fade(double t)
        {
            return t * t * t * (t * (t * 6 - 15) + 10);
        }
        // Classic Perlin noise, 3D version
        public double noise(double x, double y, double z)
        {
            // Find unit grid cell containing point
            /*
            int X = fastfloor(x);
            int Y = fastfloor(y);
            int Z = fastfloor(z);
            */
            int X = (int)(x);
            int Y = (int)(y);
            int Z = (int)(z);
              // Get relative xyz coordinates of point within that cell
            x = x - X;
            y = y - Y;
            z = z - Z;
            // Wrap the integer cells at 255 (smaller integer period can be introduced here)
            X = X & 255;
            Y = Y & 255;
            Z = Z & 255;
            // Calculate a set of eight hashed gradient indices
            int value = permBitmap.Pixels[256 * Z + Y];
            int r = (value >> 16) & 255;
            int g = (value >> 8) & 255;
            int b = value & 255;
            int a = (value >> 24) & 255;
            int gi000 = perm[X + r] % 12;//r
            int gi001 = perm[X + b] % 12;//b
            int gi010 = perm[X + g] % 12;//g
            int gi011 = perm[X + a] % 12;//a
            int gi100 = perm[X + 1 + r] % 12;
            int gi101 = perm[X + 1 + b] % 12;
            int gi110 = perm[X + 1 + g] % 12;
            int gi111 = perm[X + 1 + a] % 12;
            // The gradients of each corner are now:
            // g000 = grad3[gi000];
            // g001 = grad3[gi001];
            // g010 = grad3[gi010];
            // g011 = grad3[gi011];
            // g100 = grad3[gi100];
            // g101 = grad3[gi101];
            // g110 = grad3[gi110];
            // g111 = grad3[gi111];
            // Calculate noise contributions from each of the eight corners
            double n000 = dot(grad3[gi000], x, y, z);
            double n100 = dot(grad3[gi100], x - 1, y, z);
            double n010 = dot(grad3[gi010], x, y - 1, z);
            double n110 = dot(grad3[gi110], x - 1, y - 1, z);
            double n001 = dot(grad3[gi001], x, y, z - 1);
            double n101 = dot(grad3[gi101], x - 1, y, z - 1);
            double n011 = dot(grad3[gi011], x, y - 1, z - 1);
            double n111 = dot(grad3[gi111], x - 1, y - 1, z - 1);
            // Compute the fade curve value for each of x, y, z
            double u = fade(x);
            double v = fade(y);
            double w = fade(z);
            // Interpolate along x the contributions from each of the corners
            double nx00 = mix(n000, n100, u);
            double nx01 = mix(n001, n101, u);
            double nx10 = mix(n010, n110, u);
            double nx11 = mix(n011, n111, u);
            // Interpolate the four results along y
            double nxy0 = mix(nx00, nx10, v);
            double nxy1 = mix(nx01, nx11, v);
            // Interpolate the two last results along z
            double nxyz = mix(nxy0, nxy1, w);
            return nxyz;

        }

         // Improved Perlin noise, 2D version
        public double noise(double x, double y)
        {
            // Find unit grid cell containing point
            int X = fastfloor(x);
            int Y = fastfloor(y);
            // Get relative xyz coordinates of point within that cell
            x = x - X;
            y = y - Y;
            // Wrap the integer cells at 255 (smaller integer period can be introduced here)
            X = X & 255;
            Y = Y & 255;
            // Calculate a set of eight hashed gradient indices
            int gi000 = perm[X + perm[Y]] % 12;
            //int gi001 = perm[X + perm[Y + perm[0 + 1]]] % 12;
            int gi010 = perm[X + perm[Y + 1]] % 12;
            //int gi011 = perm[X + perm[Y + 1 + perm[0 + 1]]] % 12;
            int gi100 = perm[X + 1 + perm[Y]] % 12;
            //int gi101 = perm[X + 1 + perm[Y + perm[0 + 1]]] % 12;
            int gi110 = perm[X + 1 + perm[Y + 1]] % 12;
            //int gi111 = perm[X + 1 + perm[Y + 1 + perm[0 + 1]]] % 12;
            // The gradients of each corner are now:
            // g000 = grad3[gi000];
            // g001 = grad3[gi001];
            // g010 = grad3[gi010];
            // g011 = grad3[gi011];
            // g100 = grad3[gi100];
            // g101 = grad3[gi101];
            // g110 = grad3[gi110];
            // g111 = grad3[gi111];
            // Calculate noise contributions from each of the eight corners
            double n000 = dot(grad3[gi000], x, y, 0);
            double n100 = dot(grad3[gi100], x - 1, y, 0);
            double n010 = dot(grad3[gi010], x, y - 1, 0);
            double n110 = dot(grad3[gi110], x - 1, y - 1, 0);
            /*
            double n001 = dot(grad3[gi001], x, y,  - 1);
            double n101 = dot(grad3[gi101], x - 1, y,  - 1);
            double n011 = dot(grad3[gi011], x, y - 1,  - 1);
            double n111 = dot(grad3[gi111], x - 1, y - 1,  - 1);
            
             */ // Compute the fade curve value for each of x, y, z
            double u = fade(x);
            double v = fade(y);
            //double w = fade(z);
            // Interpolate along x the contributions from each of the corners
            double nx00 = mix(n000, n100, u);
            //double nx01 = mix(n001, n101, u);
            double nx10 = mix(n010, n110, u);
            //double nx11 = mix(n011, n111, u);
            // Interpolate the four results along y
            double nxy0 = mix(nx00, nx10, v);
           // double nxy1 = mix(nx01, nx11, v);
            // Interpolate the two last results along z
            //double nxyz = mix(nxy0, nxy1, w);
            return nxy0;
        }



        public WriteableBitmap getPerm()
        {
            WriteableBitmap oWriteableBitmap = new WriteableBitmap(256, 256);
            int index=0;
            for (int z = 0; z < 256; z++)
            {
                for (int y = 0; y < 256; y++)
                {
                    int gi000 = perm[y + perm[z]];
                    int gi001 = perm[y + perm[z + 1]];
                    int gi010 = perm[y + 1 + perm[z]];
                    int gi011 = perm[y + 1 + perm[z + 1]];
                    oWriteableBitmap.Pixels[index] = (gi011 << 24) | (gi000 << 16)| (gi001) | (gi010<<8);
                    index++;
                }
            }
            oWriteableBitmap.Invalidate();
            return oWriteableBitmap;
        }
    }
}
