﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;

namespace PerlinNoiseShader.PerlinNoise2D
{
    /// <summary>
    /// consists of methods to calculate single 2D Perlin noise ocatves
    /// </summary>
    public class PerlinNoise2DClass:NoiseMask
    {
        private ImageBrush oG2ImageBrush;
        private ImageBrush oG2ImageBrush2;
        private ImageBrush oPrepBrush;
        private ImageBrush oPrepBrush2;
        private PerlinNoiseShaderEffect[] aPerlinNoiseShaderEffect;
        private TileableEffect[] aTileableEffect;
        private WriteableBitmap oWriteableBitmap1=null, oWriteableBitmap2=null, oWriteableBitmap3=null, oWriteableBitmap4=null;
        private ImageBrush oImageBrush1, oImageBrush2, oImageBrush3, oImageBrush4;
        
        public ImageBrush g2ImageBrush
        {
            get { return oG2ImageBrush; }
        }

        #region constructors
        public PerlinNoise2DClass():this((new Random()).Next())  {}

        public PerlinNoise2DClass(int random)
        {
            oDrawRectangle = new Rectangle();
            iUIElement = oDrawRectangle;
            thisSeed=random;
            thisSize = new Size(0, 0);
            oImageBrush1 = new ImageBrush();
            oImageBrush2 = new ImageBrush();
            oImageBrush3 = new ImageBrush();
            oImageBrush4 = new ImageBrush();
            aPerlinNoiseShaderEffect = new PerlinNoiseShaderEffect[4];
            aTileableEffect = new TileableEffect[12];
            for (int i = 0; i < 4; i++)
            {
                aPerlinNoiseShaderEffect[i] = new PerlinNoiseShaderEffect((PerlinNoiseShaderEffect.state)i);
            }
            for (int i = 0; i < 12; i++)
            {
                aTileableEffect[i] = new TileableEffect((TileableEffect.state)i);
            }
            reInitialize(random);
        }

        #endregion

        #region initializing or reinitializing

        /// <summary>
        /// get the gradients
        /// </summary>
        private void getG2()
        {
            WriteableBitmap g2Bitmap = new WriteableBitmap(256, 1);
            WriteableBitmap g2Bitmap2 = new WriteableBitmap(256, 1);
            for (int i = 0; i < 256; i++)
            {
                int value = oRandom.Next() % (255 + 255 + 1) - 255;
                int value2 = oRandom.Next() % (255 + 255 + 1) - 255;
                double length = Math.Sqrt(value * value + value2 * value2);
                double v1 = value / length+1.0;
                double v2 = value2 / length+1.0;
                int v1Sign = Math.Sign(v1);
                int v2Sign = Math.Sign(v2);
                v1 = v1 < 0 ? -v1 : v1;
                v2 = v2 < 0 ? -v2 : v2;

                int r = (int)v1;
                v1 = (v1 - r) * 256;
                int g = (int)v1;
                v1 = (v1 - g) * 256;
                int b = (int)v1;
                v1 = (v1 - b) * 256;
                int a = (int)v1;
                                
                int r2 = (int)v2;
                v2 = (v2 - r2) * 256;
                int g2 = (int)v2;
                v2 = (v2 - g2) * 256;
                int b2 = (int)v2;
                v2 = (v2 - b2) * 256;
                int a2 = (int)v2;
                g2Bitmap.Pixels[i] = (a << 24) | (r << 16) | (g << 8) | b;
                g2Bitmap2.Pixels[i] = (a2 << 24) | (r2 << 16) | (g2 << 8) | b2;
            }
            g2Bitmap.Invalidate();
            g2Bitmap2.Invalidate();
            oG2ImageBrush = new ImageBrush();
            oG2ImageBrush.ImageSource = g2Bitmap;
            oG2ImageBrush2 = new ImageBrush();
            oG2ImageBrush2.ImageSource = g2Bitmap2;
         }

        /// <summary>
        /// initialize with a new random seed value
        /// </summary>
        public void reInitialize()
        {
            reInitialize((new Random()).Next());
        }
        
        /// <summary>
        /// initialize with a new random seed value
        /// </summary>
        /// <param name="random">random seed</param>
        public void reInitialize(int random)
        {
            oRandom = new Random(random);
            thisSeed = random;
            getG2();
            oPBrush = getP();
            //last preparation step
            Image oPrepImage = new Image();
            oPrepImage.Measure(new Size(256, 256));
            oPrepImage.Arrange(new Rect(0, 0, 256, 256));
            oPrepImage.Source = new WriteableBitmap(256, 256);
            PrepPerlinNoiseEffect oPrepPerlinNoiseEffect = new PrepPerlinNoiseEffect();
            oPrepImage.Effect = oPrepPerlinNoiseEffect;
            oPrepPerlinNoiseEffect.pArray = oPBrush;
            oPrepPerlinNoiseEffect.gArray = oG2ImageBrush;
            WriteableBitmap oPrepBitmap = new WriteableBitmap(256, 256);
            oPrepBrush = new ImageBrush();
            oPrepBitmap.Render(oPrepImage, null);
            oPrepBitmap.Invalidate();
            oPrepBrush.ImageSource = oPrepBitmap;
            WriteableBitmap oPrepBitmap2 = new WriteableBitmap(256, 256);
            oPrepBrush2 = new ImageBrush();
            oPrepPerlinNoiseEffect.gArray = oG2ImageBrush2;
            oPrepBitmap2.Render(oPrepImage, null);
            oPrepBitmap2.Invalidate();
            oPrepBrush2.ImageSource = oPrepBitmap2;
        }

        #endregion

        /// <summary>
        /// calculates a single octave of 2D Perlin noise
        /// </summary>
        /// <param name="freqXY">frequency at the x and y axis</param>
        /// <param name="offset">upper left corner of the plane</param>
        /// <param name="amplitude"></param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="c1">start color</param>
        /// <param name="c2">end color</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void singleNoise(Size freqXY, Size offset, float amplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if(!thisSize.Equals(testSize)){
                thisSize=testSize;
                if (oDrawRectangle != null) createRectangle();
            }
            PerlinNoiseShaderEffect oPerlinNoiseShaderEffect=absolute?this.aPerlinNoiseShaderEffect[(int)PerlinNoiseShaderEffect.state.singleAbs]:
                this.aPerlinNoiseShaderEffect[(int)PerlinNoiseShaderEffect.state.single];
            iUIElement.Effect = oPerlinNoiseShaderEffect;
            oPerlinNoiseShaderEffect.g2Array = oPrepBrush;
            oPerlinNoiseShaderEffect.g2Array2 = oPrepBrush2;
            oPerlinNoiseShaderEffect.size = new Size(freqXY.Width*thisSize.Width, freqXY.Height*thisSize.Height);
            Size tempOffset = new Size(offset.Width * freqXY.Width, offset.Height * freqXY.Height);
            while (tempOffset.Width >= 256) tempOffset.Width -= 256;
            while (tempOffset.Height >= 256) tempOffset.Height -= 256;
            
            oPerlinNoiseShaderEffect.offset = new Size(tempOffset.Width + 4096,tempOffset.Height + 4096);
            oPerlinNoiseShaderEffect.c1 = c1;
            oPerlinNoiseShaderEffect.c2 = c2;
            oPerlinNoiseShaderEffect.amplitude = absolute ? amplitude : 0.5f * amplitude;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }

        /// <summary>
        /// calculates a single octave of 2D Perlin noise to be mixed with other noise
        /// </summary>
        /// <param name="freqXY">frequency at the x and y axis</param>
        /// <param name="offset">upper left corner of the plane</param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void noise(Size freqXY, Size offset, bool absolute, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawRectangle != null) createRectangle();
            }
            PerlinNoiseShaderEffect oPerlinNoiseShaderEffect = absolute ? this.aPerlinNoiseShaderEffect[(int)PerlinNoiseShaderEffect.state.standardAbs] :
                this.aPerlinNoiseShaderEffect[(int)PerlinNoiseShaderEffect.state.standard];
            iUIElement.Effect = oPerlinNoiseShaderEffect;
            oPerlinNoiseShaderEffect.g2Array = oPrepBrush;
            oPerlinNoiseShaderEffect.g2Array2 = oPrepBrush2;
            oPerlinNoiseShaderEffect.size = new Size(freqXY.Width * thisSize.Width, freqXY.Height * thisSize.Height);
            Size tempOffset = new Size(offset.Width * freqXY.Width, offset.Height * freqXY.Height);
            while (tempOffset.Width >= 256) tempOffset.Width -= 256;
            while (tempOffset.Height >= 256) tempOffset.Height -= 256;
            oPerlinNoiseShaderEffect.offset = new Size(tempOffset.Width + 4096, tempOffset.Height + 4096);

            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }
        
        /// <summary>
        /// calculates a single octave of tileable 2D Perlin noise  
        /// </summary>
        /// <param name="freqXY">frequency at the x and y axis</param>
        /// <param name="offset">upper, left corner of the noise plane</param>
        /// <param name="amplitude"></param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="c1">start color</param>
        /// <param name="c2">end color</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void singleTileableNoise(Size freqXY, Size offset, float amplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            singleTileableNoise(freqXY, offset, amplitude, absolute, true, true, c1, c2, oWriteableBitmap);
           
        }

        /// <summary>
        /// calculates a single octave of tileable 2D Perlin noise  
        /// </summary>
        /// <param name="freqXY">frequency at the x and y axis</param>
        /// <param name="offset">upper, left corner of the noise plane</param>
        /// <param name="amplitude"></param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="tileableX">true, if the noise tiles at the width</param>
        /// <param name="tileableY">true, if the niose tiles at the height</param>
        /// <param name="c1">start color</param>
        /// <param name="c2">end color</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void singleTileableNoise(Size freqXY, Size offset, float amplitude, bool absolute, bool tileableX, bool tileableY,Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (!tileableX && !tileableY) return;
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if ((!thisSize.Equals(testSize)) || oWriteableBitmap1 == null)
            {
                oWriteableBitmap1 = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                oWriteableBitmap2 = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                oWriteableBitmap3 = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                oWriteableBitmap4 = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            }

            noise(freqXY, new Size(0, 0), false, oWriteableBitmap1);
            oImageBrush1.ImageSource = oWriteableBitmap1;
            TileableEffect.state currentState=TileableEffect.state.single;
            double tileX = -thisSize.Width;
            while (tileX < 0) tileX += 256 / freqXY.Width;
            double tileY = -thisSize.Height;
            while (tileY < 0) tileY += 256 / freqXY.Height;
            if (tileableX)
            {
                currentState = absolute ? TileableEffect.state.singleAbsX : TileableEffect.state.singleX;
                noise(freqXY, new Size(tileX, 0), false, oWriteableBitmap2);
                oImageBrush2.ImageSource = oWriteableBitmap2;
            }
            if (tileableY)
            {
                currentState = absolute ? TileableEffect.state.singleAbsY : TileableEffect.state.singleY;
                noise(freqXY, new Size(0, tileY), false, oWriteableBitmap3);
                oImageBrush3.ImageSource = oWriteableBitmap3;
            }
            if (tileableX && tileableY)
            {
                currentState = absolute ? TileableEffect.state.singleAbs : TileableEffect.state.single;
                noise(freqXY, new Size(tileX, tileY), false, oWriteableBitmap4);
                oImageBrush4.ImageSource = oWriteableBitmap4;
            }

            while (offset.Width > thisSize.Width) offset.Width -= thisSize.Width;
            while (offset.Height > thisSize.Height) offset.Height -= thisSize.Height;
            TileableEffect oTileableEffect = aTileableEffect[(int)currentState];
            iUIElement.Effect = oTileableEffect;
            oTileableEffect.input1 = oImageBrush1;
            if(tileableX)
                oTileableEffect.input2 = oImageBrush2;
            else oTileableEffect.input2 = oImageBrush3;
            if (tileableX && tileableY)
            {
                oTileableEffect.input3 = oImageBrush3;
                oTileableEffect.input4 = oImageBrush4;
            }
            oTileableEffect.size = new Size(thisSize.Width * freqXY.Width, thisSize.Height * freqXY.Height);
            oTileableEffect.offset = new Size(offset.Width / thisSize.Width, offset.Height / thisSize.Height);
            oTileableEffect.amplitude = absolute ? amplitude : 0.5f * amplitude;
            oTileableEffect.c1 = c1;
            oTileableEffect.c2 = c2;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }

        /// <summary>
        /// calculates a single octave of tileable 2D Perlin noise to be mixed with other noise
        /// </summary>
        /// <param name="freqXY">frequency at the x and y axis</param>
        /// <param name="offset">upper, left corner of the noise plane</param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void tileableNoise(Size freqXY, Size offset, bool absolute, WriteableBitmap oWriteableBitmap)
        {
            tileableNoise(freqXY, offset, absolute, true, true, oWriteableBitmap);
        }

        /// <summary>
        /// calculates a single octave of tileable 2D Perlin noise to be mixed with other noise 
        /// </summary>
        /// <param name="freqXY">frequency at the x and y axis</param>
        /// <param name="offset">upper, left corner of the noise plane</param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="tileableX">true, if the noise tiles at the width</param>
        /// <param name="tileableY">true, if the niose tiles at the height</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void tileableNoise(Size freqXY, Size offset, bool absolute, bool tileableX, bool tileableY, WriteableBitmap oWriteableBitmap)
        {
            if (!tileableX && !tileableY) return;
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if ((!thisSize.Equals(testSize)) || oWriteableBitmap1 == null)
            {
                oWriteableBitmap1 = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                oWriteableBitmap2 = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                oWriteableBitmap3 = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                oWriteableBitmap4 = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            }
            TileableEffect.state currentState = TileableEffect.state.single;
            double tileX = -thisSize.Width;
            while (tileX < 0) tileX += 256 / freqXY.Width;
            double tileY = -thisSize.Height;
            while (tileY < 0) tileY += 256 / freqXY.Height;
            if (tileableX)
            {
                currentState = absolute ? TileableEffect.state.standardAbsX : TileableEffect.state.standardX;
                noise(freqXY, new Size(tileX, 0), false, oWriteableBitmap2);
                oImageBrush2.ImageSource = oWriteableBitmap2;
            }
            if (tileableY)
            {
                currentState = absolute ? TileableEffect.state.standardAbsY : TileableEffect.state.standardY;
                noise(freqXY, new Size(0, tileY), false, oWriteableBitmap3);
                oImageBrush3.ImageSource = oWriteableBitmap3;
            }
            if (tileableX && tileableY)
            {
                currentState = absolute ? TileableEffect.state.standardAbs : TileableEffect.state.standard;
                noise(freqXY, new Size(tileX, tileY), false, oWriteableBitmap4);
                oImageBrush4.ImageSource = oWriteableBitmap4;
            }
            while (offset.Width > thisSize.Width) offset.Width -= thisSize.Width;
            while (offset.Height > thisSize.Height) offset.Height -= thisSize.Height;
            TileableEffect oTileableEffect = absolute ? aTileableEffect[3] : aTileableEffect[2];
            iUIElement.Effect = oTileableEffect;
            oTileableEffect.input1 = oImageBrush1;
            if (tileableX)
                oTileableEffect.input2 = oImageBrush2;
            else oTileableEffect.input2 = oImageBrush3;
            if (tileableX && tileableY)
            {
                oTileableEffect.input3 = oImageBrush3;
                oTileableEffect.input4 = oImageBrush4;
            } 
            oTileableEffect.size = new Size(thisSize.Width * freqXY.Width, thisSize.Height * freqXY.Height);
            oTileableEffect.offset = new Size(offset.Width / thisSize.Width, offset.Height / thisSize.Height);
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }
    }
}
