﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;

namespace PerlinNoiseShader.Simplex2D
{
    public class Simplex2DShaderEffect:ShaderEffect
    {
        public enum state { single, singleAbs, standard, standardAbs };

        private DependencyProperty pArrayProperty;
        private DependencyProperty gi1GradArrayProperty;
        private DependencyProperty gi02GradArrayProperty;
        private DependencyProperty sizeProperty;
        private DependencyProperty offsetProperty;
        private DependencyProperty c1Property;
        private DependencyProperty c2Property;
        private DependencyProperty amplitudeProperty;

        public Simplex2DShaderEffect(state eState)
        {
            pArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("pArray", typeof(Simplex2DShaderEffect), 0);
            gi1GradArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("gi1GradArray", typeof(Simplex2DShaderEffect), 1);
            gi02GradArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("gi02GradArray", typeof(Simplex2DShaderEffect), 2);
            sizeProperty = DependencyProperty.Register("size", typeof(Size), typeof(Simplex2DShaderEffect), new PropertyMetadata(new Size(0, 0), PixelShaderConstantCallback(0)));
            offsetProperty = DependencyProperty.Register("offset", typeof(Size), typeof(Simplex2DShaderEffect), new PropertyMetadata(new Size(0, 0), PixelShaderConstantCallback(1)));
            switch (eState)
            {
                case state.single:
                    {
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(Simplex2DShaderEffect), new PropertyMetadata(Color.FromArgb(0, 255, 0, 0), PixelShaderConstantCallback(2)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(Simplex2DShaderEffect), new PropertyMetadata(Color.FromArgb(0, 255, 0, 0), PixelShaderConstantCallback(3)));
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(Simplex2DShaderEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(4)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/Simplex2D/Simplex2D.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(amplitudeProperty);
                        break;
                    }
                case state.singleAbs:
                    {
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(Simplex2DShaderEffect), new PropertyMetadata(Color.FromArgb(0, 255, 0, 0), PixelShaderConstantCallback(2)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(Simplex2DShaderEffect), new PropertyMetadata(Color.FromArgb(0, 255, 0, 0), PixelShaderConstantCallback(3)));
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(Simplex2DShaderEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(4)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/Simplex2D/Simplex2DAbs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(amplitudeProperty);
                        break;
                    }
                case state.standard:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/Simplex2D/Simplex2D2.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        break;
                    }
                case state.standardAbs:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/Simplex2D/Simplex2D2Abs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        break;
                    }
            }
            
            UpdateShaderValue(pArrayProperty);
            UpdateShaderValue(gi1GradArrayProperty);
            UpdateShaderValue(gi02GradArrayProperty);
            UpdateShaderValue(sizeProperty);
            UpdateShaderValue(offsetProperty);
        }

        public Brush pArray
        {
            get { return (Brush)GetValue(pArrayProperty); }
            set { SetValue(pArrayProperty, value); }
        }

        public Brush gi1GradArray
        {
            get { return (Brush)GetValue(gi1GradArrayProperty); }
            set { SetValue(gi1GradArrayProperty, value); }
        }

        public Brush gi02GradArray
        {
            get { return (Brush)GetValue(gi02GradArrayProperty); }
            set { SetValue(gi02GradArrayProperty, value); }
        }

        public Size size
        {
            get { return (Size)GetValue(sizeProperty); }
            set { SetValue(sizeProperty, value); }
        }

        public Size offset
        {
            get { return (Size)GetValue(offsetProperty); }
            set { SetValue(offsetProperty, value); }
        }

        public float amplitude
        {
            get { return (float)GetValue(amplitudeProperty); }
            set { SetValue(amplitudeProperty, value); }
        }

        public Color c1
        {
            get { return (Color)GetValue(c1Property); }
            set { SetValue(c1Property, value); }
        }

        public Color c2
        {
            get { return (Color)GetValue(c2Property); }
            set { SetValue(c2Property, value); }
        }


    }
}
