﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using PerlinNoiseShader.ImprovedNoise;
using PerlinNoiseShader.NoiseBlender;
using System.Windows.Media.Media3D;
using System.Windows.Browser;
using PerlinProject.SphereMapping;
using PerlinNoiseShader;
using System.Collections;

namespace PerlinProject
{
    public partial class MainPage : UserControl
    {

        private int mouseClicked = -1;
        private int currentSelected = 0;
        private Point[] oldPoint = new Point[5], newPoint = new Point[5];
        private Ellipse[] aEllipse = new Ellipse[5];
        private WriteableBitmap xPlane, yPlane, zPlane, oLightMap;
        private WriteableBitmap[] aWriteableBitmap=new WriteableBitmap[5], aWriteableBitmap2=new WriteableBitmap[5];
        private ImageBrush xBrush, yBrush, zBrush, oLightMapBrush=new ImageBrush();
        private int diameter2 = 220, diameter;
        private double radius, radius2;
        private Matrix3d[] aSavedMatrix = new Matrix3d[5];
        private Matrix3d oMatrix3d = new Matrix3d();
        private SphereMappingEffect[] aSphereMappingEffect=new SphereMappingEffect[5];
        private SphereMappingLightEffect[] aSphereMappingLightEffect=new SphereMappingLightEffect[5];
        private Image[] aImage = new Image[5];
        
        public MainPage()
        {
            for (int i = 0; i < 5; i++)
            {
                oldPoint[i] = new Point();
                newPoint[i] = new Point();
                aSavedMatrix[i] = new Matrix3d();
                aEllipse[i] = new Ellipse();
            }
            InitializeComponent();
        }

        private int convertDoubleToInt(double number)
        {
            int r = (int)number;
            number = (number - r) * 256;
            int g = (int)number;
            number = (number - g) * 256;
            int b = (int)number;
            number = (number - b) * 256;
            int a = (int)number;
            return (a << 24) | (r << 16) | (g << 8) | b;
        }

  
        
        /// <summary>
        /// gets the virtual sphere coordinates from the mouse position
        /// </summary>
        /// <param name="p"></param>
        /// <returns></returns>
        private Vector3d trackBallMapping(Point p)
        {
            double x = 640 / 2.0 - p.X;
            double y = 480 / 2.0 - p.Y;
            double z2 = 640 * 480*1000 - x * x - y * y;
            double z = z2 > 0 ? Math.Sqrt(z2) : 0;
            return new Vector3d(x, y, z);
        }
       

        

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            diameter = (int)(2 * diameter2);
                        
            radius = diameter / 2.0;
            radius2 = diameter2 / 2.0;
            
            aWriteableBitmap2[0] = new WriteableBitmap(2*diameter, diameter);
            //calculates a sphere and writes the x,y and z values to xPlane, yPlane and zPlane
            //the height of the sphere map must be greater than the radius of the sphere to avoid distortions
            xPlane = new WriteableBitmap(2*diameter, diameter);
            yPlane = new WriteableBitmap(2*diameter, diameter);
            zPlane = new WriteableBitmap(2*diameter, diameter);
            int index = 0;
            double freq = 0.0265;
            freq = freq*diameter2/diameter;
            for (int y = 0; y < diameter; y++)
            {
                for (int x = 0; x < 2*diameter; x++)
                {
                    double xInsert = 2 * Math.PI * x / diameter/2;
                    double yInsert = Math.PI * (y-radius) / diameter;
                    double xValue = freq * radius * Math.Cos(yInsert) * Math.Cos(xInsert);
                    double yValue = freq * radius * Math.Cos(yInsert) * Math.Sin(xInsert);
                    double zValue = freq * radius * Math.Sin(yInsert);
                    xPlane.Pixels[index] = convertDoubleToInt(xValue+freq*radius);
                    yPlane.Pixels[index] = convertDoubleToInt(yValue+freq*radius);
                    zPlane.Pixels[index] = convertDoubleToInt(zValue+freq*radius);
                    index++;
                }
            }
                        
            xPlane.Invalidate();
            yPlane.Invalidate();
            zPlane.Invalidate();
            xBrush=new ImageBrush();
            yBrush=new ImageBrush();
            zBrush=new ImageBrush();
            xBrush.ImageSource=xPlane;
            yBrush.ImageSource=yPlane;
            zBrush.ImageSource=zPlane;

            for (int i = 0; i < 5; i++)
            {
                EllipseGeometry oEllipseGeometry = new EllipseGeometry();
                oEllipseGeometry.Center = new Point(radius2, radius2);
                oEllipseGeometry.RadiusX = radius2;
                oEllipseGeometry.RadiusY = radius2;
                aEllipse[i].Clip = oEllipseGeometry;
                aEllipse[i].Width = diameter2;
                aEllipse[i].Height = diameter2;
                aEllipse[i].Fill = new SolidColorBrush(Colors.Red);
                aEllipse[i].StrokeThickness = 0;
            }
            
            //first sphere map with single noise        
            ImprovedNoiseClass oImprovedNoiseClass = new ImprovedNoiseClass();            
            oImprovedNoiseClass.singleNoise(xBrush, yBrush, zBrush, new Vector3d(5000, 5000, 5000), 1, false, Color.FromArgb(255, 0, 0, 255), Color.FromArgb(255, 200, 200, 255), aWriteableBitmap2[0]);
            aWriteableBitmap[0] = new WriteableBitmap(diameter2, diameter2);
            //every sphere has its own rotation and its own rotation matrix
            oMatrix3d.fromAngleAxis(0 * Math.PI / 180, new Vector3d(1,0,0));
            for(int i=0;i<5;i++) aSavedMatrix[i] = oMatrix3d.mult(aSavedMatrix[i]);
          
            //for a lighted sphere, calculate the light map
            oLightMap = new WriteableBitmap(diameter2, diameter2);
            SphereLightEffect oSphereLightEffect=new SphereLightEffect();
            oSphereLightEffect.difLight = Color.FromArgb(255, 76, 76, 76);
            oSphereLightEffect.specLight = Color.FromArgb(255, 35, 35, 35);
            oSphereLightEffect.specular = 8;
            oSphereLightEffect.lightDirection = (new Vector3d(1, -1, 1)).normalize();
            aEllipse[0].Effect = oSphereLightEffect;
            oLightMap.Render(aEllipse[0], null);
            oLightMap.Invalidate();
            oLightMapBrush.ImageSource = oLightMap;
            
            //render the first sphere
            ImageBrush sourceBrush = new ImageBrush();
            sourceBrush.ImageSource = aWriteableBitmap2[0];
            aSphereMappingLightEffect[0] = new SphereMappingLightEffect();
            aSphereMappingLightEffect[0].rotMatrix = aSavedMatrix[0];
            aSphereMappingLightEffect[0].sourceBrush = sourceBrush;
            aSphereMappingLightEffect[0].lightBrush = oLightMapBrush;
            aSphereMappingEffect[0] = new SphereMappingEffect();
            aSphereMappingEffect[0].rotMatrix = aSavedMatrix[0];
            aSphereMappingEffect[0].sourceBrush = sourceBrush;
            aEllipse[0].Effect = aSphereMappingLightEffect[0];
            aWriteableBitmap[0].Render(aEllipse[0], null);
            aWriteableBitmap[0].Invalidate();
            
            myImage1.Source=aWriteableBitmap[0];
            aImage[0] = myImage1;
            aImage[1] = myImage2;
            aImage[2] = myImage3;
            aImage[3] = myImage4;
            aImage[4] = myImage5;
       }
        
        

        private void UserControl_MouseMove(object sender, MouseEventArgs e)
        {
            //rotate the current sphere
            if (mouseClicked!=-1)
            {
                newPoint[currentSelected] = e.GetPosition(null);
                Vector3d oLastVector3d = this.trackBallMapping(oldPoint[currentSelected]);

                Vector3d oNewVector3d = this.trackBallMapping(newPoint[currentSelected]);
                Vector3d oAxisVector = oLastVector3d.cross(oNewVector3d);
                double tAngle = Math.Sqrt((newPoint[currentSelected].X - oldPoint[currentSelected].X) * (newPoint[currentSelected].X - oldPoint[currentSelected].X) + 
                    (newPoint[currentSelected].Y - oldPoint[currentSelected].Y) * (newPoint[currentSelected].Y - oldPoint[currentSelected].Y));
                
                
                oMatrix3d.fromAngleAxis(tAngle * Math.PI / 180, oAxisVector);
                aSavedMatrix[currentSelected] = aSavedMatrix[currentSelected].mult(oMatrix3d);



                aSphereMappingEffect[currentSelected].rotMatrix = aSavedMatrix[currentSelected];
                aSphereMappingLightEffect[currentSelected].rotMatrix = aSavedMatrix[currentSelected];

                Array.Clear(aWriteableBitmap[currentSelected].Pixels, 0, aWriteableBitmap[currentSelected].Pixels.Length);
                //oClearBitmap.Pixels.CopyTo(aWriteableBitmap[currentSelected].Pixels, 0);
                aWriteableBitmap[currentSelected].Render(aEllipse[currentSelected], null);
                aWriteableBitmap[currentSelected].Invalidate();


                oldPoint[currentSelected].X = newPoint[currentSelected].X;
                oldPoint[currentSelected].Y = newPoint[currentSelected].Y;
            }
            
        }

        

        private void UserControl_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            mouseClicked = currentSelected;
            oldPoint[currentSelected] = e.GetPosition(null);
            newPoint[currentSelected] = e.GetPosition(null);
        }

        private void UserControl_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            aSavedMatrix[currentSelected] = oMatrix3d.mult(aSavedMatrix[currentSelected]);
            mouseClicked = -1;
        }

        private void UserControl_MouseLeave(object sender, MouseEventArgs e)
        {
            mouseClicked = -1;
        }

        
        /// <summary>
        /// selects the sphere and if not precalculated, create a sphere map
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TabControl1_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            currentSelected = ((TabControl)sender).SelectedIndex;
            if (aWriteableBitmap[currentSelected] == null && currentSelected>0)
            {
                this.Cursor = Cursors.Wait;
                aWriteableBitmap[currentSelected] = new WriteableBitmap(diameter2, diameter2);
                aWriteableBitmap2[currentSelected] = new WriteableBitmap(2 * diameter, diameter);
                switch (currentSelected)
                {
                    case 1://|noise|
                        {
                            ImprovedNoiseClass oImprovedNoiseClass = new ImprovedNoiseClass();
                            oImprovedNoiseClass.singleNoise(xBrush, yBrush, zBrush, new Vector3d(5000, 5000, 5000), 1, true, Color.FromArgb(255, 0, 0, 255), Color.FromArgb(255, 200, 200, 255), aWriteableBitmap2[currentSelected]);
                            break;
                        }
                    case 2://sum(noise)
                        {
                            NoiseBlenderClass oNoiseBlenderClass = new NoiseBlenderClass();
                            int octaves = 6;
                            Vector3d[] aOffset = new Vector3d[octaves];
                            float[] aAmplitude = new float[octaves];
                            Vector3d[] aMult = new Vector3d[octaves];
                            
                            float ampValue = 1;// 1 / 0.2f;
                            float value = 1f;
                            for (int i = 0; i < octaves; i++)
                            {
                                aOffset[i] = new Vector3d(5000, 5000,5000);
                                aAmplitude[i] = ampValue;
                                aMult[i] = new Vector3d(value, value, value);
                                value *= 2.0f;
                                ampValue /= 2.0f;
                            }
                            oNoiseBlenderClass.improvedNoises(xBrush, yBrush, zBrush, octaves, aMult, aOffset, aAmplitude, false, Color.FromArgb(255, 0, 0, 255), Color.FromArgb(255, 200, 200, 255), aWriteableBitmap2[currentSelected]);
                            break;
                        }
                    case 3://sum(|noise|)
                        {
                            NoiseBlenderClass oNoiseBlenderClass = new NoiseBlenderClass();
                            int octaves = 6;
                            Vector3d[] aOffset = new Vector3d[octaves];
                            float[] aAmplitude = new float[octaves];
                            Vector3d[] aMult = new Vector3d[octaves];

                            float ampValue = 1;// 1 / 0.2f;
                            float value = 1f;
                            for (int i = 0; i < octaves; i++)
                            {
                                aOffset[i] = new Vector3d(5000, 5000, 5000);
                                aAmplitude[i] = ampValue;
                                aMult[i] = new Vector3d(value, value, value);
                                value *= 2.0f;
                                ampValue /= 2.0f;
                            }
                            oNoiseBlenderClass.improvedNoises(xBrush, yBrush, zBrush, octaves, aMult, aOffset, aAmplitude, true, Color.FromArgb(255, 0, 0, 255), Color.FromArgb(255, 200, 200, 255), aWriteableBitmap2[currentSelected]);
                            break;
                        }
                    case 4://cos(x+sum(|noise|))=>marble
                        {
                            NoiseBlenderClass oNoiseBlenderClass = new NoiseBlenderClass();
                            int octaves = 6;
                            Vector3d[] aOffset = new Vector3d[octaves];
                            float[] aAmplitude = new float[octaves];
                            Vector3d[] aMult = new Vector3d[octaves];

                            float ampValue = 10;// 1..10
                            float value = 1f;
                            for (int i = 0; i < octaves; i++)
                            {
                                aOffset[i] = new Vector3d(5000, 5000, 5000);
                                aAmplitude[i] = ampValue;
                                aMult[i] = new Vector3d(value, value, value);
                                value *= 2.0f;
                                ampValue /= 1.5f;
                            }
                            oNoiseBlenderClass.improvedNoises(xBrush, yBrush, zBrush, octaves, aMult, aOffset, aAmplitude, 0.8f, new Vector3d(5, 0, 0), 0, Color.FromArgb(255, 0, 0, 255), Color.FromArgb(255, 200, 200, 255), aWriteableBitmap2[currentSelected]);
                            break;
                        } 
                    
                    default://noise
                        {
                            ImprovedNoiseClass oImprovedNoiseClass = new ImprovedNoiseClass();
                            oImprovedNoiseClass.singleNoise(xBrush, yBrush, zBrush, new Vector3d(5000, 5000, 5000), 1, false, Color.FromArgb(255, 0, 0, 255), Color.FromArgb(255, 200, 200, 255), aWriteableBitmap2[currentSelected]);
                            break;
                        }
                }
                
                ImageBrush sourceBrush = new ImageBrush();
                sourceBrush.ImageSource = aWriteableBitmap2[currentSelected];
                aSphereMappingLightEffect[currentSelected] = new SphereMappingLightEffect();
                aSphereMappingLightEffect[currentSelected].rotMatrix = aSavedMatrix[currentSelected];
                aSphereMappingLightEffect[currentSelected].sourceBrush = sourceBrush;
                aSphereMappingLightEffect[currentSelected].lightBrush = oLightMapBrush;
                aSphereMappingEffect[currentSelected] = new SphereMappingEffect();
                aSphereMappingEffect[currentSelected].rotMatrix = aSavedMatrix[currentSelected];
                aSphereMappingEffect[currentSelected].sourceBrush = sourceBrush;
                aEllipse[currentSelected].Effect = aSphereMappingLightEffect[currentSelected];
                aWriteableBitmap[currentSelected].Render(aEllipse[currentSelected], null);
                
                aWriteableBitmap[currentSelected].Invalidate();
                
                aImage[currentSelected].Source = aWriteableBitmap[currentSelected];
                this.Cursor = Cursors.Arrow;
            }
        }

        /// <summary>
        /// switches the light on and off
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void checkBox_Click(object sender, RoutedEventArgs e)
        {
            if (((CheckBox)sender).IsChecked == true)
            {
                aEllipse[currentSelected].Effect = aSphereMappingLightEffect[currentSelected];
            }
            else
            {
                aEllipse[currentSelected].Effect = aSphereMappingEffect[currentSelected];
            }
            Array.Clear(aWriteableBitmap[currentSelected].Pixels, 0, aWriteableBitmap[currentSelected].Pixels.Length);
            aWriteableBitmap[currentSelected].Render(aEllipse[currentSelected], null);
            aWriteableBitmap[currentSelected].Invalidate();
        }

        
    }
}
