﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using VoxelSpacingTop.Shader;
using PerlinNoiseShader.NoiseBlender;
using PerlinNoiseShader.ImprovedNoise2D;

namespace VoxelSpacingTop
{
    public partial class MainPage : UserControl
    {
        private const int WIDTH = 600;
        private const int HEIGHT = 400;
        private const int DEPTH = 400;

        WriteableBitmap oWriteableBitmap, geoMap, illumMap, shadowMap;
        private const int maxLength = 378;
        private double angle = 0;
        private double xp = 512, yp = 800;
        private int hp = 330;
        private int whichMap = 2;
        private int color = 2;
        private EventHandler animation;
        int step=0;
        private int[] screenIndex = new int[WIDTH * HEIGHT];
        int[] buffer = new Int32[maxLength * 2048];

        public MainPage()
        {
            InitializeComponent();
            this.KeyDown += new KeyEventHandler(MainPage_KeyDown);
            animation=new EventHandler(CompositionTarget_Rendering);
            CompositionTarget.Rendering += animation;
        }

        void CompositionTarget_Rendering(object sender, EventArgs e)
        {
            if (step == 0)
            {
                this.label1.Content = "Calculating landscape...";
                step++;
            }
            else if (step == 1)
            {
                //calculate landscape
                geoMap = new WriteableBitmap(1024, 1024);
                NoiseBlenderClass oNoiseBlenderClass = new NoiseBlenderClass();
                oNoiseBlenderClass.turbulence2D(new Size(8.0 / 1024.0, 8.0 / 1024.0), new iSize(8, 8), 7, /*random,*/ null, 1, 0.5f, false, Color.FromArgb(0, 0, 0, 0), Color.FromArgb(255, 255, 255, 255), geoMap);
                this.label1.Content = "Calculating illumination...";
                step++;
            }
            else if (step == 2)
            {
                //calculate the light map
                illumMap = new WriteableBitmap(1024, 1024);
                Rectangle illumRectangle = new Rectangle();
                illumRectangle.Width = 1024;
                illumRectangle.Height = 1024;
                illumRectangle.Fill = new SolidColorBrush(Colors.Red);
                IllumShader oIllumShader = new IllumShader();
                oIllumShader.geoMap = geoMap;
                illumRectangle.Effect = oIllumShader;
                illumMap.Render(illumRectangle, null);
                //illumination replaced by a shader
                /*
                int index = 0; 
                Vector3d light=new Vector3d(2,0,-1);
                light=light.normalize();
                for (int y = 0; y < 1024; y++)
                {
                    for (int x = 0; x < 1024; x++)
                    {
                        int z1 = geoMap.Pixels[1024 * y + x] & 255;
                        int z2 = geoMap.Pixels[1024 * y + ((x - 1 + 1024) & 1023)] & 255;
                        int z3 = geoMap.Pixels[1024 * ((y - 1+1024) & 1023) + x] & 255;
                        Vector3d v1 = new Vector3d(1, 0, z1-z2);
                        Vector3d v2 = new Vector3d(0, 1, z1-z3);
                    
                        v1 = v1.normalize();
                        v2 = v2.normalize();
                        Vector3d v3 = v2.cross(v1);
                        v3 = v3.normalize();
                        double result = v3.dot(light);
                        if (result < 0) result = 0;
                        if (result > 1) result = 1;
                        int color = (int)(0.5*(255 * result+z1));
                        illumMap.Pixels[index] = z1 << 24 | color << 16 | color << 8 | color;
                        index++;
                    }
                }
                */

                illumMap.Invalidate();
                this.label1.Content = "Calculating shadow map...";
                step++;
            }
            else if (step == 3)
            {
                //calculate the shadow
                shadowMap = new WriteableBitmap(1024, 1024);
                int index = 0;
                //for every pixel at the bitmap: follow back the light vector. if there is an object before reaching the maximum height,
                //there must be a shadow for the current pixel
                for (int y = 0; y < 1024; y++)
                {
                    for (int x = 0; x < 1024; x++)
                    {
                        int z1 = geoMap.Pixels[index] & 255;
                        int color = illumMap.Pixels[index] & 255;
                        double height = z1;
                        int index2 = 1;
                        while (height < 255)
                        {
                            height += 2;
                            int z2 = geoMap.Pixels[1024 * y + ((x - index2 + 1024) & 1023)] & 255;
                            if (height <= z2)
                            {
                                color = 120 * z1 / 255 / 3;
                                break;
                            }
                            index2++;
                        }
                        shadowMap.Pixels[index] = z1 << 24 | color << 16 | color << 8 | color;
                        index++;
                    }
                }
                shadowMap.Invalidate();
                oWriteableBitmap = new WriteableBitmap(WIDTH, HEIGHT);
                this.label1.Content = "";
                this.label1.Visibility = Visibility.Collapsed;
                screenTransfer();
                drawLandscape();
                myImage.Source = oWriteableBitmap;
                step++;
            }
            else if (step == 4) CompositionTarget.Rendering -= animation;
            
        }

        private void screenTransfer()
        {
            for (int y = 0; y < HEIGHT; y++)
            {
                for (int x = 0; x < WIDTH; x++)
                {
                    double xpos = x - (WIDTH - 1) / 2.0;
                    double ypos = y - (HEIGHT - 1) / 2.0;
                    double length = Math.Sqrt(xpos * xpos + ypos * ypos);
                    int xIndex = (int)length;
                    double atan = (Math.Atan2(ypos, xpos) * 1024 / Math.PI);
                    if (atan < 0) atan += 2048; else if (atan > 2048) atan -= 2048;
                    int yIndex = (int)atan;
                    screenIndex[WIDTH * y + x] = yIndex * maxLength + xIndex;
                }
            }
        }

        //fixed point version, faster
        private void drawLandscape()
        {
            WriteableBitmap readMap;
            if (whichMap == 0)
            {
                readMap = geoMap;
            }
            else if (whichMap == 1)
            {
                readMap = illumMap;
            }
            else
            {
                readMap = shadowMap;
            }
            int selectColor;
            if (color == 0)
            {
                selectColor = 0xFF0000;
            }
            else if (color == 1)
            {
                selectColor = 0xFF00;
            }
            else
            {
                selectColor = 0xFF;
            }
            for (int i = 0; i < 2048; i++)
            {
                double dx = Math.Cos(i * Math.PI / 1024 + angle * Math.PI / 180);
                double dy = Math.Sin(i * Math.PI / 1024 + angle * Math.PI / 180);
                double x1 = xp;
                double y1 = yp;
                int index = ((int)y1 & 1023) * 1024 + ((int)x1 & 1023);
                int ci = readMap.Pixels[index];
                buffer[i * maxLength] = (ci & selectColor) | (255 << 24);
                x1 += dx;
                y1 += dy;
                double length = Math.Sqrt((x1 - xp) * (x1 - xp) + (y1 - yp) * (y1 - yp));
                int ix1 = (int)(65536 * x1);
                int iy1 = (int)(65536 * y1);
                int idx = (int)(65536 * dx);
                int idy = (int)(65536 * dy);
                int iLength = (int)(1024 * length);
                int currentFill = 0;
                int count = 0;
                while (count < maxLength)
                {
                    index = ((iy1>>16) & 1023) * 1024 + ((ix1>>16) & 1023);
                    ci = readMap.Pixels[index];
                    int pixel = (ci & selectColor) | (255 << 24);
                    int h = (ci >> 24) & 255;
                    int divi = hp - h;
                    count = divi <= 0 ? Int32.MaxValue : (640 * iLength / divi)>>10;
                    for (int fill = currentFill; fill < count && fill < maxLength; fill++, currentFill = fill)
                    {
                        buffer[i * maxLength + fill] = pixel;
                    }
                    ix1 += idx; iy1 += idy;
                    iLength+=1024;

                }
            }
            for (int i = 0; i < oWriteableBitmap.Pixels.Length; i++)
            {
                oWriteableBitmap.Pixels[i] = buffer[screenIndex[i]];
            }
            oWriteableBitmap.Invalidate();
        }

        /*//floating point version
        private void drawLandscape()
        {
            WriteableBitmap readMap;
            if (whichMap == 0)
            {
                readMap = geoMap;
            }
            else if (whichMap == 1)
            {
                readMap = illumMap;
            }
            else
            {
                readMap = shadowMap;
            }
            int selectColor;
            if (color == 0)
            {
                selectColor = 0xFF0000;
            }
            else if (color == 1)
            {
                selectColor = 0xFF00;
            }
            else
            {
                selectColor = 0xFF;
            }
            for (int i = 0; i < 2048; i++)
            {
                double dx = Math.Cos(i * Math.PI / 1024 + angle * Math.PI / 180);
                double dy = Math.Sin(i * Math.PI / 1024 + angle * Math.PI / 180);
                double x1 = xp;
                double y1 = yp;
                int index = ((int)y1 & 1023) * 1024 + ((int)x1 & 1023);
                int ci = readMap.Pixels[index];
                buffer[i * maxLength] = (ci & selectColor) | (255 << 24); 
                x1 += dx;
                y1 += dy;
                double length = Math.Sqrt((x1 - xp) * (x1 - xp) + (y1 - yp) * (y1 - yp));
                
                int currentFill = 0;
                int count = 0;
                while (count < maxLength)
                {
                    index = ((int)y1 & 1023) * 1024 + ((int)x1 & 1023);
                    ci = readMap.Pixels[index];
                    int pixel = (ci & selectColor) | (255 << 24);
                    int h = (ci >> 24) & 255;
                    int divi = hp - h;
                    count = divi <= 0 ? Int32.MaxValue : (int)((640) * length / divi);
                    for (int fill = currentFill; fill < count && fill < maxLength; fill++, currentFill = fill)
                    {
                        buffer[i * maxLength + fill] = pixel;
                    }
                    x1 += dx; y1 += dy;
                    length++;

                }
            }
            for (int i = 0; i < oWriteableBitmap.Pixels.Length; i++)
            {
                oWriteableBitmap.Pixels[i] = buffer[screenIndex[i]];
            }
            oWriteableBitmap.Invalidate();
        }
        */
        void MainPage_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Left:
                    {
                        xp -= 2 * Math.Cos(angle * Math.PI / 180);
                        yp -= 2 * Math.Sin(angle * Math.PI / 180);
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.Right:
                    {
                        xp += 2 * Math.Cos(angle * Math.PI / 180);
                        yp += 2 * Math.Sin(angle * Math.PI / 180);
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.Up:
                    {
                        yp -= 2 * Math.Cos(-angle * Math.PI / 180);
                        xp -= 2 * Math.Sin(-angle * Math.PI / 180); 
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.Down:
                    {
                        yp += 2 * Math.Cos(-angle * Math.PI / 180);
                        xp += 2 * Math.Sin(-angle * Math.PI / 180);
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.W:
                    {
                        //move up
                        hp += 2;
                        if (hp > 512) hp = 512;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.S:
                    {
                        //move down
                        hp -= 2;
                        if (hp < 0) hp = 0;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.A:
                    {
                        //rotation
                        angle += 5;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.D:
                    {
                        //rotation
                        angle -= 5;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.Space:
                    {
                        //change the maps: normal, lighted, lighted + shadow
                        whichMap++;
                        if (whichMap > 2) whichMap = 0;
                        drawLandscape();
                        e.Handled = true;
                        break;
                    }
                case Key.R:
                    {
                        //red
                        color = 0;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.G:
                    {
                        //green
                        color = 1;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.B:
                    {
                        //blue
                        color = 2;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }

            }
        }
    }
}
